#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..util_core.group import Vmess, Socks, Mtproto, SS
from ..util_core.writer import ClientWriter, GroupWriter
from ..util_core.selector import ClientSelector, GroupSelector

def alterid():
    cs = ClientSelector('modify alterId')
    group = cs.group

    if group == None:
        pass
    else:
        client_index = cs.client_index
        if type(group.node_list[client_index]) == Vmess:
            print("node alterID: {}".format(group.node_list[client_index].alter_id))
            new_alterid = input("please input new alterID: ")
            if (new_alterid.isnumeric()):
                cw = ClientWriter(group.tag, group.index, client_index)
                cw.write_aid(int(new_alterid))
                print("alterID modify success!")
            else:
                print ("input error, please check is number")
        else:
            print("only vmess protocol can modify alterId!")

def dyn_port():
    gs = GroupSelector('modify dyn_port')
    group = gs.group

    if group == None:
        pass
    else:
        print('dyn_port status: {}'.format(group.dyp))
        gw = GroupWriter(group.tag, group.index)
        
        choice = input("open/close dyn_port(y/n): ").lower()

        if choice == 'y':
            newAlterId = input("please input dyn_port alterID(default 32): ")
            newAlterId = '32' if newAlterId == '' else newAlterId
            if (newAlterId.isdecimal()):
                gw.write_dyp(True, newAlterId)
                print("\nopen dyn_port success!")
            else:
                print ("\ninput error, please check is number")
        elif choice == 'n':
            gw.write_dyp(False)
            print("\nclose dyn_port success!")
        else:
            print ("\ninput error, please try again")

def new_email():
    cs = ClientSelector('modify email')
    group = cs.group

    if group == None:
        pass
    elif type(group.node_list[0]) == Socks:
        print("Socks5 don't support email!")
    else:
        client_index = cs.client_index
        group_list = cs.group_list
        print ("node email: {}".format(group.node_list[client_index].user_info))
        email = ""
        while True:
            is_duplicate_email=False
            email = input("please input new email: ")
            if email == "":
                break
            from ..util_core.utils import is_email
            if not is_email(email):
                print("not email, please input again")
                continue
            
            for loop_group in group_list:
                for node in loop_group.node_list:
                    if node.user_info == None or node.user_info == '':
                        continue
                    elif node.user_info == email:
                        print("have same email, please input other")
                        is_duplicate_email = True
                        break              
            if not is_duplicate_email:
                break

        if email != "":
            cw = ClientWriter(group.tag, group.index, client_index)
            cw.write_email(email)
            print("modify email success!")

def new_uuid():
    cs = ClientSelector('modify uuid')
    group = cs.group

    if group == None:
        pass
    else:
        client_index = cs.client_index
        if type(group.node_list[client_index]) == Vmess:
            print("node UUID: {}".format(group.node_list[client_index].password))
            choice = input("get new UUID?(y/n): ").lower()
            if choice == "y":
                import uuid
                new_uuid = uuid.uuid1()
                print("new UUID: {}".format(new_uuid))
                cw = ClientWriter(group.tag, group.index, client_index)
                cw.write_uuid(new_uuid)
                print("UUID modify success!")
            else:
                print("undo modify")
        else:
            print("only vmess protocol can modify uuid!")

def port():
    gs = GroupSelector('modify port')
    group = gs.group

    if group == None:
        pass
    else:
        if group.end_port:
            port_info = "{0}-{1}".format(group.port, group.end_port)
        else:
            port_info = group.port
        print('group port: {}'.format(port_info))
        new_port_info = input("please input new port(support range port(use '-' as separator), all range port can effect):")
        import re
        if new_port_info.isdecimal() or re.match(r'^\d+\-\d+$', new_port_info):
            gw = GroupWriter(group.tag, group.index)
            gw.write_port(new_port_info)
            print('port modify success!')
        else:
            print("input error!")

def tfo():
    gs = GroupSelector('modify tcpFastOpen')
    group = gs.group

    if group == None:
        exit(-1)
    else:
        if type(group.node_list[0]) == Mtproto or type(group.node_list[0]) == SS:
            print("\nv2ray MTProto/Shadowsocks don't support tcpFastOpen!!!\n")
            exit(-1)
        
        print('group tcpFastOpen: {}'.format(group.tfo))
        print("")
        print("1.open TFO(force open)")
        print("2.close TFO(force close)")
        print("3.delete TFO(use system default profile)")
        choice = input("please select: ")
        
        gw = GroupWriter(group.tag, group.index)
        if choice == "1":
            gw.write_tfo('on')
        elif choice == "2":
            gw.write_tfo('off')
        elif choice == "3":
            gw.write_tfo('del')
        else:
            print("input error, please try again!")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json
import os
from flask import request, jsonify, Blueprint

from .util_core.loader import Loader
from .util_core.utils import ss_method, stream_list, StreamType
from .util_core.writer import NodeWriter, GroupWriter, ClientWriter, GlobalWriter, StreamWriter

func_router = Blueprint('func_router', __name__)

loader = Loader()

class ResponseJson:
    def __init__(self, success=True, msg="success", data=None):
        self.success = success
        self.msg = msg
        self.data = data

def find_client(group_list, client_index):
    find, group = False, None
    for single_group in group_list:
        if find:
            break
        for index, node in enumerate(single_group.node_list):
            if node.user_number == client_index:
                client_index = index
                group = single_group
                find = True
                break
    return group, client_index

@func_router.route('/list', methods=['GET'])
def node_list():
    loader.load_profile()
    return json.dumps(loader.profile, default=lambda x: x.__dict__, ensure_ascii=False)

@func_router.route('/collection/<ctype>', methods=['GET'])
def collection(ctype):
    success, msg, result = True, "get {} success!!!".format(ctype), None
    try:
        if ctype == 'ss_method':
            result = ss_method()
        elif ctype == 'new_port':
            result = [x.value for x in stream_list()]
        elif ctype == 'stream':
            result = [x.value for x in StreamType]
        else:
            raise ValueError("{} ctype not found".format(ctype))
    except Exception as e:
        success = False
        msg = str(e)
    return jsonify(ResponseJson(success, msg, result).__dict__)

@func_router.route('/manage/<action>', methods=['POST'])
def manage(action):
    success, msg, result = True, "{} v2ray success!!!".format(action), None
    try:
        if action in ['start', 'stop', 'restart']:
            os.system("service v2ray {}".format(action))
        else:
            raise ValueError("{} action not found".format(action))
    except Exception as e:
        success = False
        msg = str(e)
    return jsonify(ResponseJson(success, msg, result).__dict__)

@func_router.route('/user', methods = ['POST'])
def add_user():
    success, msg, kw = True, "add user success!!!", dict()
    try:
        json_request = json.loads(request.get_data())
        group_tag = json_request['group_tag']
        kw['email'] = json_request['email']
        loader.load_profile()
        group_list = loader.profile.group_list
        group = list(filter(lambda group:group.tag == group_tag, group_list))[0]
        nw = NodeWriter(group.tag, group.index)
        nw.create_new_user(**kw)
    except Exception as e:
        success = False
        msg = str(e)
    return jsonify(ResponseJson(success, msg).__dict__)

@func_router.route('/user/<int:client_index>', methods = ['DELETE'])
def del_user(client_index):
    success, msg = True, "del user {} success!!!".format(client_index)
    try:
        loader.load_profile()
        group_list = loader.profile.group_list
        group, client_index = find_client(group_list, client_index)
        nw = NodeWriter()
        nw.del_user(group, client_index)
    except Exception as e:
        success = False
        msg = str(e)
    return jsonify(ResponseJson(success, msg).__dict__)

@func_router.route('/group', methods = ['POST'])
def add_group():
    success, msg, kw, stream_type = True, "add group {} success!!!", dict(), ""
    try:
        json_request = json.loads(request.get_data())
        port = int(json_request['port'])
        stream_type = json_request['stream_type']

        from .util_core.utils import stream_list
        stream_list = stream_list()
        if stream_type not in [x.value for x in stream_list]:
            raise ValueError("stream_type {} not found".format(stream_type))
        if "data" in json_request:
            kw = json_request['data']
        
        stream = list(filter(lambda stream:stream.value == stream_type, stream_list))[0]
        nw = NodeWriter()
        nw.create_new_port(port, stream, **kw)
        msg = msg.format(stream_type)
    except Exception as e:
        success = False
        msg = str(e)
    return jsonify(ResponseJson(success, msg).__dict__)

@func_router.route('/group/<group_tag>', methods = ['DELETE'])
def del_group(group_tag):
    success, msg = True, "del group {} success!!!".format(group_tag)
    try:
        loader.load_profile()
        group_list = loader.profile.group_list
        group = list(filter(lambda group:group.tag == group_tag, group_list))[0]
        nw = NodeWriter()
        nw.del_port(group)
    except Exception as e:
        success = False
        msg = str(e)
    return jsonify(ResponseJson(success, msg).__dict__)

@func_router.route('/stream/<group_tag>', methods = ['PUT'])
def modify_stream(group_tag):
    success, msg, kw = True, "modify group {} success!!!".format(group_tag), dict()
    try:
        json_request = json.loads(request.get_data())
        stream_type = json_request['stream_type']
        if stream_type not in [x.value for x in StreamType]:
            raise ValueError("stream_type {} not found".format(stream_type))
        if "data" in json_request:
            kw = json_request['data']
        stream = list(filter(lambda stream:stream.value == stream_type, StreamType))[0]

        loader.load_profile()
        group_list = loader.profile.group_list
        group = list(filter(lambda group:group.tag == group_tag, group_list))[0]
        sw = StreamWriter(group.tag, group.index, stream)
        sw.write(**kw)
    except Exception as e:
        success = False
        msg = str(e)
    return jsonify(ResponseJson(success, msg).__dict__)

@func_router.route('/group/<group_tag>', methods = ['PUT'])
def modify_group(group_tag):
    success, msg = True, "modify group {} success!!!".format(group_tag)
    try:
        json_request = json.loads(request.get_data())
        modify_type = json_request['modify_type']
        value = json_request['value']
        loader.load_profile()
        group_list = loader.profile.group_list
        group = list(filter(lambda group:group.tag == group_tag, group_list))[0]
        gw = GroupWriter(group.tag, group.index)
        method = 'write_' + modify_type
        if hasattr(gw, method):
            func = getattr(gw, method)
            if modify_type == "dyp":
                func(bool(value['status']), int(value['aid']))
            else:
                func(value)
        else:
            raise RuntimeError("{} method not found".format(method))
    except Exception as e:
        success = False
        msg = str(e)
    return jsonify(ResponseJson(success, msg).__dict__)

@func_router.route('/user/<int:client_index>', methods = ['PUT'])
def modify_user(client_index):
    success, msg, modify_type = True, "modify user {} success!!!".format(client_index), ""
    try:
        json_request = json.loads(request.get_data())
        modify_type = json_request['modify_type']
        value = json_request['value']
        loader.load_profile()
        group_list = loader.profile.group_list
        group, client_index = find_client(group_list, client_index)
        cw = ClientWriter(group.tag, group.index, client_index)
        method = 'write_' + modify_type
        if hasattr(cw, method):
            func = getattr(cw, method)
            func(value)
        else:
            raise RuntimeError("{} method not found".format(method))
    except Exception as e:
        success = False
        msg = str(e)
    return jsonify(ResponseJson(success, msg).__dict__)

@func_router.route('/global', methods = ['PUT'])
def modify_global():
    success, msg, modify_type = True, "modify global {} success!!!", ""
    try:
        json_request = json.loads(request.get_data())
        modify_type = json_request['modify_type']
        value = json_request['value']
        loader.load_profile()
        group_list = loader.profile.group_list
        gw = GlobalWriter(group_list)
        method = 'write_' + modify_type
        if hasattr(gw, method):
            func = getattr(gw, method)
            func(value)
        else:
            raise RuntimeError("{} method not found".format(method))
        msg = msg.format(modify_type)
    except Exception as e:
        success = False
        msg = str(e)
    return jsonify(ResponseJson(success, msg).__dict__)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..util_core.loader import Loader
from ..util_core.writer import GlobalWriter

def manage():
    loader = Loader()

    profile = loader.profile

    print("Ban BT status: {}".format(profile.ban_bt))

    choice = input("Ban BT?(y/n)").lower()

    ban_bt = True if choice == 'y' else False

    gw = GlobalWriter(profile.group_list)

    gw.write_ban_bittorrent(ban_bt)

    print("modify success!")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from flask import Flask, request, jsonify, render_template
from flask_httpauth import HTTPBasicAuth

from .app import func_router
from .util_core.config import Config

config = Config()
app = Flask(__name__, static_url_path='/static')
app.register_blueprint(func_router)

auth = HTTPBasicAuth()

@auth.get_password
def get_pw(username):
    return config.get_web('pass')

@app.route('/')
@auth.login_required
def index():
    return render_template(config.get_web('index_page'))

if __name__ == '__main__':
    app.run(debug=True, port=config.get_web('port'))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json

config_path = "/etc/v2ray/config.json"

class ConfigConverter:
    def __init__(self):
        self.config = self.load()
    
    def load(self):
        '''
        load v2ray profile
        '''
        with open(config_path, 'r') as json_file:
            config = json.load(json_file)
        return config

    def save(self):
        '''
        save v2ray config.json
        '''
        json_dump=json.dumps(self.config, indent=2)
        with open(config_path, 'w') as writer:
            writer.writelines(json_dump)

    def transform(self):
        inbound_list, outbound_list = [], []
        if "inbound" in self.config:
            inbound_list.append(self.config["inbound"])
            del self.config["inbound"]
        if "inboundDetour" in self.config and self.config["inboundDetour"]:
            inbound_list.extend([ x for x in self.config["inboundDetour"] ])
            del self.config["inboundDetour"]
        if inbound_list:
            self.config["inbounds"] = inbound_list

        if "outbound" in self.config:
            outbound_list.append(self.config["outbound"])
            del self.config["outbound"]
        if "outboundDetour" in self.config and self.config["outboundDetour"]:
            outbound_list.extend([ x for x in self.config["outboundDetour"] ])
            del self.config["outboundDetour"]
        if outbound_list:
            self.config["outbounds"] = outbound_list

        # 转换路由
        if "routing" in self.config and "settings" in self.config["routing"]:
            self.config["routing"]["rules"] = self.config["routing"]["settings"]["rules"]
            del self.config["routing"]["strategy"]
            del self.config["routing"]["settings"]
            
        self.save()

print("tranfrom to v2ray new version json..")
ConfigConverter().transform()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import subprocess
import pkg_resources

from .util_core.v2ray import V2ray
from .util_core.utils import ColorStr, open_port
from .global_setting import stats_ctr, iptables_ctr, ban_bt
from .config_modify import base, multiple, ss, stream, tls

def loop_input_choice_number(input_tip, number_max):
    """
    循环输入选择的序号,直到符合规定为止
    """
    while True:
        print("")
        choice = input(input_tip)
        if not choice:
            break
        if choice.isnumeric():
            choice = int(choice)
        else:
            print(ColorStr.red("input error, please input again"))
            continue
        if (choice <= number_max and choice > 0):
            return choice
        else:
            print(ColorStr.red("input error, please input again"))

def help():
    exec_name = sys.argv[0]
    print("""
{0} [-h|--help] [options]
    -h, --help           get help
    start                start V2Ray
    stop                 stop V2Ray
    restart              restart V2Ray
    status               check V2Ray status
    new                  create new json profile
    update               update v2ray to latest
    add                  random create mkcp + (srtp | wechat-video | utp | dtls) fake header group
    add [wechat|utp|srtp|dtls|wireguard|socks|mtproto|ss]     create special protocol, random new port
    del                  delete port group
    info                 check v2ray profile
    port                 modify port
    tls                  modify tls
    tfo                  modify tcpFastOpen
    stream               modify protocol
    stats                iptables traffic statistics
    clean                clean v2ray log
    """.format(exec_name[exec_name.rfind("/") + 1:]))

def parse_arg():
    if len(sys.argv) == 1:
        return
    elif len(sys.argv) == 2:
        if sys.argv[1] == "start":
            V2ray.start()
        elif sys.argv[1] == "stop":
            V2ray.stop()
        elif sys.argv[1] == "restart":
            V2ray.restart()
        elif sys.argv[1] == "-h" or sys.argv[1] == "--help":
            help()
        elif sys.argv[1] == "status":
            V2ray.status()
        elif sys.argv[1] == "info":
            V2ray.info()
        elif sys.argv[1] == "port":
            base.port()
            open_port()
            V2ray.restart()
        elif sys.argv[1] == "tls":
            tls.modify()
            V2ray.restart()
        elif sys.argv[1] == "tfo":
            base.tfo()
            V2ray.restart()
        elif sys.argv[1] == "stream":
            stream.modify()
            V2ray.restart()
        elif sys.argv[1] == "stats":
            iptables_ctr.manage()
        elif sys.argv[1] == "clean":
            V2ray.cleanLog()
        elif sys.argv[1] == "del":
            multiple.del_port()
            V2ray.restart()
        elif sys.argv[1] == "add":
            multiple.new_port()
            open_port()
            V2ray.restart()
        elif sys.argv[1] == "update":
            V2ray.update()
        elif sys.argv[1] == "new":
            V2ray.new()
        elif sys.argv[1] == "convert":
            V2ray.convert()
    else:
        if sys.argv[1] == "add":
            multiple.new_port(sys.argv[2])
            V2ray.restart()
    sys.exit(0)

def service_manage():
    show_text = ("start v2ray", "stop v2ray", "restart v2ray", "v2ray status")
    print("")
    for index, text in enumerate(show_text): 
        print("{}.{}".format(index + 1, text))
    choice = loop_input_choice_number("please select: ", len(show_text))
    if choice == 1:
        V2ray.start()
    elif choice == 2:
        V2ray.stop()
    elif choice == 3:
        V2ray.restart()
    elif choice == 4:
        V2ray.status()

def user_manage():
    show_text = ("add user", "add port", "del user", "del port")
    print("")
    for index, text in enumerate(show_text): 
        print("{}.{}".format(index + 1, text))
    choice = loop_input_choice_number("please select: ", len(show_text))
    if choice == 1:
        multiple.new_user()
    elif choice == 2:
        multiple.new_port()
        open_port()
    elif choice == 3:
        multiple.del_user()
    elif choice == 4:
        multiple.del_port()
    V2ray.restart()

def profile_alter():
    show_text = ("email", "UUID", "alterID", "port", "stream", "tls", 
                "tcpFastOpen", "dyn_port", "shadowsocks method", "shadowsocks password")
    print("")
    for index, text in enumerate(show_text): 
        print("{}.{}".format(index + 1, text))
    choice = loop_input_choice_number("please select: ", len(show_text))
    if choice == 1:
        base.new_email()
    elif choice == 2:
        base.new_uuid()
    elif choice == 3:
        base.alterid()
    elif choice == 4:
        base.port()
        open_port()
    elif choice == 5:
        stream.modify()
    elif choice == 6:
        tls.modify()
    elif choice == 7:
        base.tfo()
    elif choice == 8:
        base.dyn_port()
    elif choice == 9:
        ss.modify('method')
    elif choice == 10:
        ss.modify('password')
    V2ray.restart()

def global_setting():
    show_text = ("V2ray Traffic Statistics", "Iptables Traffic Statistics", "Ban Bittorrent", "Schedule Update V2ray", "Clean Log")
    print("")
    for index, text in enumerate(show_text): 
        print("{}.{}".format(index + 1, text))
    choice = loop_input_choice_number("please select: ", len(show_text))
    if choice == 1:
        stats_ctr.manage()
    elif choice == 2:
        iptables_ctr.manage()
    elif choice == 3:
        ban_bt.manage()
        V2ray.restart()
    elif choice == 4:
        subprocess.call("bash {0}".format(pkg_resources.resource_filename(__name__, "global_setting/update_timer.sh")), shell=True)
    elif choice == 5:
        V2ray.cleanLog()

def menu():
    V2ray.check()
    parse_arg()
    while True:
        print("")
        print(ColorStr.cyan("Welcome to v2ray-util"))
        print("")
        show_text = ("1.V2ray Manage", "2.Group Manage", "3.Modify Config", "4.Check Config", "5.Global Setting", "6.Update V2Ray", "7.Generate Client Json")
        for index, text in enumerate(show_text): 
            if index % 2 == 0:
                print('{:<20}'.format(text), end="")   
            else:
                print(text)
                print("")
        print("")
        choice = loop_input_choice_number("please select: ", len(show_text))
        if choice == 1:
            service_manage()
        elif choice == 2:
            user_manage()
        elif choice == 3:
            profile_alter()
        elif choice == 4:
            V2ray.info()
        elif choice == 5:
            global_setting()
        elif choice == 6:
            V2ray.update()
        elif choice == 7:
            from .util_core import client
            client.generate()
        else:
            break

if __name__ == "__main__":
    menu()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import pickle

from .config import Config
from .profile import Profile

class Loader:
    def __init__(self):
        config = Config()
        self.config_path = config.get_path("config_path")
        self.path = config.data_path
        self.profile = None
        self.load_profile()

    def load_profile(self):
        try:
            if os.path.exists(self.path):
                with open(self.path, 'rb') as reader:
                    self.profile = pickle.load(reader)
                if os.path.getmtime(self.profile.path) != self.profile.modify_time:
                    raise ValueError
            else:
                raise FileNotFoundError
        except Exception:
            self.profile = Profile()
            self.save_profile()

    def save_profile(self):
        with open(self.path, 'wb') as writer:
            pickle.dump(self.profile, writer)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import random
import sys

from ..util_core.writer import NodeWriter, GroupWriter
from ..util_core.group import Vmess, Socks, Mtproto, SS
from ..util_core.selector import GroupSelector, ClientSelector
from ..util_core.utils import StreamType, stream_list, is_email, clean_iptables, ColorStr

def new_port(new_stream=None):
    info = dict()
    if new_stream:
        correct_list = stream_list()
        if new_stream not in [x.value for x in correct_list]:
            print("input error! input -h or --help to get help")
            exit(-1)
        
        stream = list(filter(lambda stream:stream.value == new_stream, correct_list))[0]

        if stream == StreamType.SOCKS:
            user = input("please input socks user: ")
            password = input("please input socks password: ")
            if user == "" or password == "":
                print("socks user or password is null!!")
                exit(-1)
            info = {"user":user, "pass": password}
        elif stream == StreamType.SS:
            from .ss import SSFactory
            sf = SSFactory()
            info = {"method": sf.get_method(), "password": sf.get_password()}
    else:
        salt_stream = [StreamType.KCP_DTLS, StreamType.KCP_WECHAT, StreamType.KCP_UTP, StreamType.KCP_SRTP]
        random.shuffle(salt_stream)
        stream = salt_stream[0]
        print("random generate (srtp | wechat-video | utp | dtls) fake header, new protocol: {} \n".format(ColorStr.green(stream.value)))

    random_port = random.randint(1000, 65535)
    new_port = input("random generate port {}, enter to use, or input customize port: ".format(ColorStr.green(str(random_port))))

    if not new_port:
        new_port = str(random_port)

    if new_port.isnumeric():
        print("\nnew port: {} \n".format(new_port))
        nw = NodeWriter()
        nw.create_new_port(int(new_port), stream, **info)
    else:
        print ("\ninput error, please input number!!")

def new_user():
    gs = GroupSelector('user number')
    group = gs.group
    group_list = gs.group_list

    if group == None:
        exit(-1)
    else:
        email = ""
        if type(group.node_list[0]) == Vmess: 
            while True:
                is_duplicate_email=False

                email = input("input email to create user, or enter to pass: ")
                if email == "":
                    break
                if not is_email(email):
                    print("not email, please input again")
                    continue
                
                for loop_group in group_list:
                    for node in loop_group.node_list:
                        if node.user_info == None or node.user_info == '':
                            continue
                        elif node.user_info == email:
                            print("have same email, please input other")
                            is_duplicate_email = True
                            break              
                if not is_duplicate_email:
                    break

            nw = NodeWriter(group.tag, group.index)
            info = {'email': email}
            nw.create_new_user(**info)

        elif type(group.node_list[0]) == Socks:
            print("local group is socks, please input user and password to create user\n")
            user = input("please input socks user: ")
            password = input("please input socks password: ")
            if user == "" or password == "":
                print("socks user or password is null!!!")
                exit(-1)
            info = {"user":user, "pass": password}
            nw = NodeWriter(group.tag, group.index)
            nw.create_new_user(**info)

        elif type(group.node_list[0]) == Mtproto:
            print("\nMtproto protocol only support one user!!")

        elif type(group.node_list[0]) == SS:
            print("\nShadowsocks protocol only support one user, u can add new port to multiple SS!!")

def del_port():
    gs = GroupSelector('del port')
    group = gs.group

    if group == None:
        pass
    else:
        print("del group info: ")
        print(group)
        choice = input("delete?(y/n): ").lower()
        if choice == 'y':
            nw = NodeWriter()
            nw.del_port(group)
            clean_iptables(group.port)
        else:
            print("undo delete")

def del_user():
    cs = ClientSelector('del user')
    group = cs.group

    if group == None:
        pass
    else:
        client_index = cs.client_index
        print("del user info:")
        print(group.show_node(client_index))
        choice = input("delete?(y/n):").lower()
        if choice == 'y':
            if len(group.node_list) == 1:
                clean_iptables(group.port)
            nw = NodeWriter()
            nw.del_user(group, client_index)
        else:
            print("undo delete")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import subprocess
import pkg_resources

from ..util_core.loader import Loader
from ..util_core.utils import ColorStr, calcul_iptables_traffic

def manage():
    loader = Loader()

    profile = loader.profile

    group_list = profile.group_list

    while True:
        print("")
        print("Iptables Traffic Statistics")
        print("")
        print("1.check statistics result\n")
        print("2.reset special port statistics\n")

        choice = input("please select:")
        if choice == "1":
            print("")
            for group in group_list:
                print(calcul_iptables_traffic(group.port))
            print("")

        elif choice == "2":
            port = input("please input reset port:")
            if port and port.isnumeric():
                subprocess.call("bash {0} {1}".format(pkg_resources.resource_filename(__name__, "clean_traffic.sh"), str(port)), shell=True)
                print(ColorStr.green("reset success!"))
            else:
                print(ColorStr.red("input error!"))
        else:
            break


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import socket
import os

from ..util_core.writer import GroupWriter
from ..util_core.group import Mtproto, SS
from ..util_core.selector import GroupSelector
from ..util_core.utils import get_ip, gen_cert

class TLSModifier:
    def __init__(self, group_tag, group_index):
        self.writer = GroupWriter(group_tag, group_index)
    
    def turn_on(self):
        print("1. Let's Encrypt certificate(auto create, please prepare domain)")
        print("2. Customize certificate(prepare certificate file paths)\n")
        choice=input("please select: ")
        if choice == "1":
            local_ip = get_ip()
            print("local vps ip address: " + local_ip + "\n")
            input_domain=input("please input your vps domain: ")
            try:
                input_ip = socket.gethostbyname(input_domain)
            except Exception:
                print("\ndomain check error!!!\n")
                return
            if input_ip != local_ip:
                print("\ndomain can't analysis to local ip!!!\n")
                return

            print("")
            print("auto generate SSL certificate, please wait..")
            gen_cert(input_domain)
            crt_file = "/root/.acme.sh/" + input_domain +"_ecc"+ "/fullchain.cer"
            key_file = "/root/.acme.sh/" + input_domain +"_ecc"+ "/"+ input_domain +".key"

            self.writer.write_tls(True, crt_file=crt_file, key_file=key_file, domain=input_domain)

        elif choice == "2":
            crt_file = input("please input certificate cert file path: ")
            key_file = input("please input certificate key file path: ")
            if not os.path.exists(crt_file) or not os.path.exists(key_file):
                print("certificate cert or key not exist!\n")
                return
            domain = input("please input the certificate cert file domain: ")
            if not domain:
                print("domain is null!\n")
                return
            self.writer.write_tls(True, crt_file=crt_file, key_file=key_file, domain=domain)
        else:
            print("input error!\n")
    
    def turn_off(self):
        self.writer.write_tls(False)

def modify():
    gs = GroupSelector('modify TLS')
    group = gs.group

    if group == None:
        exit(-1)
    else:
        if type(group.node_list[0]) == Mtproto or type(group.node_list[0]) == SS:
            print("\nv2ray MTProto/Shadowsocks protocol not support https!!!\n")
            exit(-1)
        tm = TLSModifier(group.tag, group.index)
        tls_status = 'open' if group.tls == 'tls' else 'close'
        print("group tls status: {}\n".format(tls_status))
        print("")
        print("1.open TLS")
        print("2.close TLS")
        choice = input("please select: ")
        if choice == '1':
            tm.turn_on()
        elif choice == '2':
            tm.turn_off()
        else:
            print("input error, please try again!")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import uuid
import random
import subprocess
import pkg_resources
from .utils import ColorStr, open_port

class V2ray:

    @staticmethod
    def run(command, keyword):
        try:
            subprocess.check_output(command, shell=True)
            open_port()
            print(ColorStr.green("v2ray {} success !".format(keyword)))
        except subprocess.CalledProcessError:
            print(ColorStr.red("v2ray {} fail !".format(keyword)))

    @staticmethod
    def status():
        subprocess.call("service v2ray status", shell=True)

    @staticmethod
    def info():
        from .loader import Loader 
        print(Loader().profile)

    @staticmethod
    def update():
        subprocess.Popen("curl -L -s https://install.direct/go.sh|bash", shell=True).wait()

    @staticmethod
    def cleanLog():
        subprocess.call("cat /dev/null > /var/log/v2ray/access.log", shell=True)
        subprocess.call("cat /dev/null > /var/log/v2ray/error.log", shell=True)
        print(ColorStr.green("clean v2ray log success!\n"))

    @classmethod
    def restart(cls):
        cls.run("service v2ray restart", "restart")

    @classmethod
    def start(cls):
        cls.run("service v2ray start", "start")

    @classmethod
    def stop(cls):
        cls.run("service v2ray stop", "stop")

    @classmethod
    def convert(cls):
        from .converter import ConfigConverter

    @classmethod
    def check(cls):
        if not os.path.exists("/usr/bin/v2ray/v2ray"):
            print(ColorStr.yellow("check v2ray no install, auto install v2ray.."))
            cls.update()
            cls.new()
        if not os.path.exists("/etc/v2ray_util/util.cfg"):
            subprocess.call("mkdir -p /etc/v2ray_util && cp -f {} /etc/v2ray_util/".format(pkg_resources.resource_filename(__name__, 'util.cfg')), shell=True)

    @classmethod
    def new(cls):
        subprocess.call("rm -rf /etc/v2ray/config.json && cp {}/server.json /etc/v2ray/config.json".format(pkg_resources.resource_filename('v2ray_util', "json_template")), shell=True)
        new_uuid = uuid.uuid1()
        print("new UUID: {}".format(ColorStr.green(str(new_uuid))))
        random_port = random.randint(1000, 65535)
        print("new port: {}".format(ColorStr.green(str(random_port))))
        subprocess.call("sed -i \"s/cc4f8d5b-967b-4557-a4b6-bde92965bc27/{0}/g\" /etc/v2ray/config.json && sed -i \"s/999999999/{1}/g\" /etc/v2ray/config.json".format(new_uuid, random_port), shell=True)
        from ..config_modify import stream
        stream.StreamModifier().random_kcp()
        cls.restart()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import re

from ..util_core.loader import Loader
from ..util_core.writer import GlobalWriter
from ..util_core.utils import bytes_2_human_readable, ColorStr

class StatsFactory:
    def __init__(self, door_port):
        self.door_port = door_port
        self.downlink_value = 0
        self.uplink_value = 0

    def get_stats(self, meta_info, is_reset=False, is_group=False):
        is_reset = "true" if is_reset else "false"

        stats_cmd = "cd /usr/bin/v2ray && ./v2ctl api --server=127.0.0.1:%s StatsService.GetStats 'name: \"%s>>>%s>>>traffic>>>%s\" reset: %s'"
        type_tag = ("inbound" if is_group else "user")

        stats_real_cmd = stats_cmd % (str(self.door_port), type_tag, meta_info, "downlink", is_reset)
        downlink_result = os.popen(stats_real_cmd).readlines()
        if downlink_result and len(downlink_result) == 5:
            re_result = re.findall(r"\d+", downlink_result[2])
            if not re_result:
                print("no data traffic now!")
                return
            self.downlink_value = int(re_result[0])

        stats_real_cmd = stats_cmd % (str(self.door_port), type_tag, meta_info, "uplink", is_reset)
        uplink_result = os.popen(stats_real_cmd).readlines()
        if uplink_result and len(uplink_result) == 5:
            re_result = re.findall(r"\d+", uplink_result[2])
            self.uplink_value = int(re_result[0])

    def print_stats(self):
        print('''
downlink: {0}  
uplink: {1} 
total: {2}
        '''.format(ColorStr.cyan(bytes_2_human_readable(self.downlink_value, 2)),
        ColorStr.cyan(bytes_2_human_readable(self.uplink_value, 2)),
        ColorStr.cyan(bytes_2_human_readable(self.downlink_value + self.uplink_value, 2)))
        )


def manage():

    RESTART_CMD = "service v2ray restart"

    FIND_V2RAY_CRONTAB_CMD = "crontab -l|grep v2ray"

    DEL_UPDATE_TIMER_CMD = "crontab -l|sed '/SHELL=/d;/v2ray/d' > crontab.txt && crontab crontab.txt >/dev/null 2>&1 && rm -f crontab.txt >/dev/null 2>&1"

    while True:
        loader = Loader()

        profile = loader.profile

        group_list = profile.group_list

        print("V2ray Traffic Statistics: {}".format(profile.stats.status))

        print("")
        print("1.open statistics\n")
        print("2.close statistics\n")
        print("3.check statistics result\n")
        print("4.reset statistics\n")
        print("tip: only have email node can statistics, restart v2ray will reset traffic statistics!!!\n")

        choice = input("please select: ")
        if choice == "1":
            if os.popen(FIND_V2RAY_CRONTAB_CMD).readlines():
                rchoice = input("open traffic statistics will close schedule update v2ray, continue?(y/n): ")
                if rchoice == "y" or rchoice == "Y":
                    #关闭定时更新v2ray服务
                    os.system(DEL_UPDATE_TIMER_CMD)
                else:
                    print("undo open traffic statistics!!")
                    continue
            gw = GlobalWriter(group_list)
            gw.write_stats(True)
            os.system(RESTART_CMD)
            print("open traffic statistics success!\n")
            
        elif choice == "2":
            gw = GlobalWriter(group_list)
            gw.write_stats(False)
            os.system(RESTART_CMD)
            print("close traffic statistics success!\n")
        elif choice == "3" or choice == "4":
            is_reset = (False if choice == "3" else True)
            action_info = ("check" if choice == "3" else "reset")
            if not profile.stats.status:
                print("only open traffic statistics can {}\n".format(action_info))
                continue
            
            if group_list[-1].node_list[-1].user_number > 1:
                print(profile)
                schoice = input("please input number or group alphabet to {}: ".format(action_info))
            else:
                schoice = "A"

            sf = StatsFactory(profile.stats.door_port)
            schoice=schoice.upper()
            if schoice.isnumeric() :
                schoice = int(schoice)
                if schoice > 0 and schoice <= group_list[-1].node_list[-1].user_number:
                    find = False
                    for group in group_list:
                        if find:
                            break
                        for index, node in enumerate(group.node_list):
                            if node.user_number == schoice:
                                if node.user_info:
                                   sf.get_stats(node.user_info, is_reset)
                                   sf.print_stats()
                                else:
                                    print("no effective email!!!\n")
                                find = True
                                break
            elif schoice.isalpha() and schoice <= group_list[-1].tag:
                sf.get_stats(schoice, is_reset, True)
                sf.print_stats()
            else:
                print("input error, please input again\n")
        else:
            break


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from .loader import Loader
from .utils import ColorStr

class CommonSelector:
    def __init__(self, collection, msg):
        from collections.abc import Iterable
        if isinstance(collection, Iterable):
            self.collection = collection
        else:
            raise ValueError("{} object can't iterate".format(collection))
        self.msg = msg
    
    def select(self):
        for index, element in enumerate(self.collection):
            print("{0}.{1}".format(index + 1, element))
        choice = input(self.msg)

        if not choice.isnumeric():
            raise RuntimeError('input error, please check is number')

        choice = int(choice)
        if choice < 1 or choice > len(self.collection):
            raise RuntimeError('input error, input index out of range')
        else:
            return self.collection[choice - 1]

class Selector:
    def __init__(self, action):
        loader = Loader()
        self.profile = loader.profile
        self.group_list = loader.profile.group_list
        self.action = action

class ClientSelector(Selector):
    def __init__(self, action):
        super(ClientSelector, self).__init__(action)
        self.list_size = self.group_list[-1].node_list[-1].user_number
        if "del" in action and self.list_size == 1:
            print(ColorStr.red("last node can't delete!!!"))
            self.group = None
        elif self.list_size > 1: 
            self.select_client()
        else:
            self.group = self.group_list[0]
            self.client_index = 0

    def select_client(self):
        print(self.profile)
        self.group = None
        choice = input("please input number to {}: ".format(self.action))

        if not choice.isnumeric():
            print(ColorStr.red('input error, please check is number'))
            return

        choice = int(choice)
        if choice < 1 or choice > self.list_size:
            print(ColorStr.red('input error, input index out of range'))
        else:
            find = False
            for group in self.group_list:
                if find:
                    break
                for index, node in enumerate(group.node_list):
                    if node.user_number == choice:
                        self.client_index = index
                        self.group = group
                        find = True
                        break

class GroupSelector(Selector):
    def __init__(self, action):
        super(GroupSelector, self).__init__(action)
        if "del" in action and len(self.group_list) == 1:
            print(ColorStr.red("last group can't delete!!!"))
            self.group = None
        elif len(self.group_list) > 1:
            self.select_group()
        else:
            self.group = self.group_list[0]

    def select_group(self):
        print(self.profile)
        choice = input("please input group alphabet to {}: ".format(self.action))
        group_list = [x for x in self.group_list if x.tag == choice]
        if len(group_list) == 0:
            print(ColorStr.red('input error, please check group {} exist'.format(choice)))
            self.group = None
        else:
            self.group = group_list[0]


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

import v2ray_util

with open("README.md", "r", encoding='UTF-8') as fh:
    long_description = fh.read()

setup(
    name='v2ray-util',
    version=v2ray_util.__version__,
    description="a tool to manage v2ray config json",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords='python v2ray multi-v2ray vmess socks5',
    author='Jrohy',
    author_email='euvkzx@gmail.com',
    url='https://github.com/Jrohy/multi-v2ray',
    license='GPL',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    python_requires='>=3',
    entry_points={
        'console_scripts': [
            'v2ray-util = v2ray_util.main:menu'
        ]
    },
    classifiers=[
        'Topic :: Utilities',
        'Development Status :: 5 - Production/Stable',
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ]
)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import random
import string
import sys

from ..util_core.group import SS
from ..util_core.writer import GroupWriter
from ..util_core.selector import GroupSelector
from ..util_core.utils import ss_method, ColorStr

class SSFactory:
    def __init__(self):
        self.method_tuple = ss_method()

    def get_method(self):
        print ("please select shadowsocks method:")
        for index, method in enumerate(self.method_tuple):
            print ("{}.{}".format(index + 1, method))
        choice = input()
        choice = int(choice)
        if choice < 0 or choice > len(self.method_tuple):
            print("input out of range!!")
            exit(-1)
        else:
            return self.method_tuple[choice - 1]

    def get_password(self):
        random_pass = ''.join(random.sample(string.ascii_letters + string.digits, 16))
        new_pass =input("random generate password {}, enter to use, or input customize password: ".format(ColorStr.green(random_pass)))
        if not new_pass:
            new_pass = random_pass
        return new_pass

def modify(alter_type='method'):
    # 外部传参来决定修改哪种, 默认修改method
    correct_way = ("method", "password")

    if alter_type not in correct_way:
        print("input error!")
        exit(-1)

    gs = GroupSelector('modify SS')
    group = gs.group

    if group == None:
        exit(-1)
    elif type(group.node_list[0]) != SS:
        print("\nlocal group not Shadowsocks protocol!\n")
        exit(-1)
    else:
        sm = SSFactory()
        gw = GroupWriter(group.tag, group.index)
        if alter_type == correct_way[0]:
            gw.write_ss_method(sm.get_method())
        elif alter_type == correct_way[1]:
            gw.write_ss_password(sm.get_password())
        print("modify Shadowsocks {} success!\n".format(alter_type))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import random
import string

from ..util_core.writer import StreamWriter
from ..util_core.selector import GroupSelector, CommonSelector
from ..util_core.group import Mtproto, SS
from ..util_core.utils import StreamType, header_type_list, ColorStr

from .ss import SSFactory

class StreamModifier:
    def __init__(self, group_tag='A', group_index=-1):
        self.stream_type = [
            (StreamType.TCP, "TCP"), 
            (StreamType.TCP_HOST, "Fake HTTP"), 
            (StreamType.WS, "WebSocket"), 
            (StreamType.KCP, "mKCP"), 
            (StreamType.KCP_SRTP, "mKCP + srtp"), 
            (StreamType.KCP_UTP, "mKCP + utp"), 
            (StreamType.KCP_WECHAT, "mKCP + wechat-video"),
            (StreamType.KCP_DTLS, "mKCP + dtls"), 
            (StreamType.KCP_WG, "mKCP + wireguard"), 
            (StreamType.H2, "HTTP/2"), 
            (StreamType.SOCKS, "Socks5"), 
            (StreamType.MTPROTO, "MTProto"), 
            (StreamType.SS, "Shadowsocks"),
            (StreamType.QUIC, "Quic")
        ]
        self.group_tag = group_tag
        self.group_index = group_index

    def select(self, index):
        sw = StreamWriter(self.group_tag, self.group_index, self.stream_type[index][0])
        kw = {}
        if index == 0 or (index >= 3 and index <= 9) or index == 11:
            pass
        elif index == 1 or index == 2:
            host = input("please input fake domain: ")
            kw['host'] = host
        elif index == 10:
            user = input("please input socks user: ")
            password = input("please input socks password: ")
            if user == "" or password == "":
                print("socks user or password is null")
                exit(-1)
            kw = {'user': user, 'pass': password}
        elif index == 12:
            sf = SSFactory()
            kw = {"method": sf.get_method(), "password": sf.get_password()}
        elif index == 13:
            key = ""
            security_list = ('none', "aes-128-gcm", "chacha20-poly1305")
            print("")
            security = CommonSelector(security_list, "please select ss method: ").select()
            if security != "none":
                key = ''.join(random.sample(string.ascii_letters + string.digits, 8))
                new_pass = input('random generate password {}, enter to use, or input customize password: '.format(key))
                if new_pass:
                    key = new_pass
                    
            print("")
            header = CommonSelector(header_type_list(), "please select fake header: ").select()
            kw = {'security': security, 'key': key, 'header': header}
        sw.write(**kw)

    def random_kcp(self):
        kcp_list = ('mKCP + srtp', 'mKCP + utp', 'mKCP + wechat-video', 'mKCP + dtls')
        choice = random.randint(4, 7)
        print("random generate (srtp | wechat-video | utp | dtls) fake header, new protocol: {} \n".format(ColorStr.green(kcp_list[choice - 4])))
        self.select(choice)

def modify():
    gs = GroupSelector('modify protocol')
    group = gs.group

    if group == None:
        exit(-1)
    else:
        sm = StreamModifier(group.tag, group.index)

        print("group protocol: {}".format(group.node_list[0].stream()))
        print ("")
        for index, stream_type in enumerate(sm.stream_type):
            print("{0}.{1}".format(index + 1, stream_type[1]))

        choice = input()

        if not choice.isdecimal():
            print("please input number!")
        else:
            choice = int(choice)
            if choice > 0 and choice <= len(sm.stream_type):
                if (sm.stream_type[choice - 1][1] == "MTProto" or sm.stream_type[choice - 1][1] == "Shadowsocks") and group.tls == 'tls':
                    print("v2ray MTProto/Shadowsocks not support https, close tls success!")
                sm.select(choice - 1)
                print("modify protocol success")
            else:
                print("input out of range!!")